﻿Imports System.Runtime.CompilerServices
Module DataTableExtensions
	Public Enum PivotAction
		Sum = 1     'Sum up the data
        Min = 2     'Find Minimum Value
        Max = 3     'Find Maximum Value
        Count = 4   'Count Number of Occurences
    End Enum

	<Extension()>
	Public Function Pivot(ByVal SourceTable As DataTable,
								 ByVal KeyColumnName As String,
								 ByVal PivotColumnName As String,
								 ByVal ValueColumnName As String,
								 ByVal PivotAction As PivotAction) As DataTable

		'Results DataTable
		Dim ResultsData As New DataTable

		Dim Dv As DataView = SourceTable.DefaultView
		'Sort Source table by PivotColumnname
		Dv.Sort = PivotColumnName
		Dim keys(0) As DataColumn

		'KeyColumnName Values in ResultsData table must be unique
		Dim PrimaryColType As Type = SourceTable.Columns(KeyColumnName).DataType

		'Add KeyColumn in ResultData table
		ResultsData.Columns.Add(New DataColumn(KeyColumnName, PrimaryColType))

		'Loop through the rows in SourceTable and check PivotColumnName
		For i As Integer = 0 To Dv.Count - 1
			'If Column does not exist in the ResultsDataTable, add it
			If (ResultsData.Columns.Contains(Dv.Item(i).Item(PivotColumnName)) = False) Then
				ResultsData.Columns.Add(New DataColumn(Dv.Item(i).Item(PivotColumnName).ToString,
													   SourceTable.Columns(ValueColumnName).DataType))
			End If
		Next

		'Sort the SourceTable by KeyColumn
		Dv.Sort = KeyColumnName

		'Set PrimaryKey on The KeyColumn in ResultsTable 
		keys(0) = ResultsData.Columns(0)
		ResultsData.PrimaryKey = keys
		SourceTable = Dv.ToTable
		Dim dr As DataRow

		For i = 0 To SourceTable.Rows.Count - 1
			If (ResultsData.Rows.Find(SourceTable.Rows(i).Item(KeyColumnName)) Is Nothing) Then
                'If the DataRow with specified Column Does Not exist, we must add it
                dr = ResultsData.NewRow
				dr.Item(KeyColumnName) = SourceTable.Rows(i).Item(KeyColumnName)
				dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) = SourceTable.Rows(i).Item(ValueColumnName)
				ResultsData.Rows.Add(dr)
			Else
                'We retrieve the DataRow in question and update/insert pivoted values
                dr = ResultsData.Rows.Find(SourceTable.Rows(i).Item(KeyColumnName))
				If dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) Is DBNull.Value Then
                    'This is the first time we write to a cell
                    Select Case PivotAction
						'If PivotAction.Count, set value to 1
						Case PivotAction.Count
							dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) = 1
						Case Else
							'Set the value to the value in the SourceTable cell
							dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) =
								SourceTable.Rows(i).Item(ValueColumnName)

					End Select
				Else
                    'This is not the first time we write to a cell, so we update the values
                    'according to the PivotAction specified
                    Select Case PivotAction
						Case PivotAction.Count
							'If PivotAction.Count, increment by 1
							dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) =
								dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) + 1
						Case PivotAction.Max
							'If PivotAction.max, set value to Maximum of current value and the value in SourceTable
							If (SourceTable.Rows(i).Item(ValueColumnName) >
								 dr.Item(SourceTable.Rows(i).Item(PivotColumnName))) Then
								dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) =
									SourceTable.Rows(i).Item(ValueColumnName)
							End If
						Case PivotAction.Min
							'If PivotAction.Min, set value to Minimum of current value and the value in SourceTable
							If (SourceTable.Rows(i).Item(ValueColumnName) <
								 dr.Item(SourceTable.Rows(i).Item(PivotColumnName))) Then
								dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) =
									SourceTable.Rows(i).Item(ValueColumnName)
							End If
						Case PivotAction.Sum
							'If PivotAction.Sum, Add the value in the SourceDataColumn to the value in the ResultsTable
							dr.Item(SourceTable.Rows(i).Item(PivotColumnName)) +=
								SourceTable.Rows(i).Item(ValueColumnName)
					End Select
				End If
			End If
		Next

		'Clear DataView
		Dv = Nothing

		'Return Result
		Return ResultsData
	End Function
End Module
